@@ -9,20 +9,23 @@
   const { writableStreamClose, Deferred } = window.__bootstrap.streams;
   const { DOMException } = window.__bootstrap.domException;
   const { add, remove } = window.__bootstrap.abortSignal;
+  const { headersFromHeaderList, headerListFromHeaders, fillHeaders } =
+    window.__bootstrap.headers;
 
   const {
+    ArrayPrototypeJoin,
+    ArrayPrototypeMap,
+    Error,
+    ObjectPrototypeIsPrototypeOf,
+    PromisePrototypeCatch,
+    PromisePrototypeThen,
+    Set,
     StringPrototypeEndsWith,
     StringPrototypeToLowerCase,
     Symbol,
     SymbolFor,
-    Set,
-    ArrayPrototypeMap,
-    ArrayPrototypeJoin,
-    PromisePrototypeThen,
-    PromisePrototypeCatch,
-    Uint8Array,
     TypeError,
-    Error,
+    Uint8ArrayPrototype,
   } = window.__bootstrap.primordials;
 
   webidl.converters.WebSocketStreamOptions = webidl.createDictionaryConverter(
@@ -70,7 +73,7 @@
 
     [_url];
     get url() {
-      webidl.assertBranded(this, WebSocketStream);
+      webidl.assertBranded(this, WebSocketStreamPrototype);
       return this[_url];
     }
 
@@ -120,6 +123,11 @@
         );
       }
 
+      const headers = headersFromHeaderList([], "request");
+      if (options.headers !== undefined) {
+        fillHeaders(headers, options.headers);
+      }
+
       const cancelRid = core.opSync(
         "op_ws_check_permission_and_cancel_handle",
         this[_url],
@@ -143,7 +151,7 @@
               ? ArrayPrototypeJoin(options.protocols, ", ")
               : "",
             cancelHandle: cancelRid,
-            headers: [...new Headers(options.headers).entries()],
+            headers: headerListFromHeaders(headers),
           }),
           (create) => {
             options.signal?.[remove](abort);
@@ -195,7 +203,9 @@
                       kind: "text",
                       value: chunk,
                     });
-                  } else if (chunk instanceof Uint8Array) {
+                  } else if (
+                    ObjectPrototypeIsPrototypeOf(Uint8ArrayPrototype, chunk)
+                  ) {
                     await core.opAsync("op_ws_send", this[_rid], {
                       kind: "binary",
                       value: chunk,
@@ -296,7 +306,7 @@
             }
           },
           (err) => {
-            if (err instanceof core.Interrupted) {
+            if (ObjectPrototypeIsPrototypeOf(core.InterruptedPrototype, err)) {
               // The signal was aborted.
               err = options.signal.reason;
             } else {
@@ -311,19 +321,19 @@
 
     [_connection] = new Deferred();
     get connection() {
-      webidl.assertBranded(this, WebSocketStream);
+      webidl.assertBranded(this, WebSocketStreamPrototype);
       return this[_connection].promise;
     }
 
     [_earlyClose] = false;
     [_closed] = new Deferred();
     get closed() {
-      webidl.assertBranded(this, WebSocketStream);
+      webidl.assertBranded(this, WebSocketStreamPrototype);
       return this[_closed].promise;
     }
 
     close(closeInfo) {
-      webidl.assertBranded(this, WebSocketStream);
+      webidl.assertBranded(this, WebSocketStreamPrototype);
       closeInfo = webidl.converters.WebSocketCloseInfo(closeInfo, {
         prefix: "Failed to execute 'close' on 'WebSocketStream'",
         context: "Argument 1",
@@ -381,5 +391,7 @@
     }
   }
 
+  const WebSocketStreamPrototype = WebSocketStream.prototype;
+
   window.__bootstrap.webSocket.WebSocketStream = WebSocketStream;
 })(this);
